from .embed import *
from .http_ import *
from .member import *
from .message import *


class Channel:

    """
    Класс представляющий информацию о канале.
    """

    def __init__(self, channel: dict, token: str):

        self.channel: dict = channel
        self.token: str = token

    @property
    def id(self) -> str:

        return str(self.channel['id'])

    @property
    def member(self) -> Member:

        return Member(self.channel['recipients'][0])
    
    @property
    def messages(self) -> list[Message]:
        
        user_messages: list[Message] = []

        for message in HttpClient().request("GET", 'https://discordapp.com/api/v9/channels/{0}/messages'.format(self.id),
                                            {"Authorization": self.token, "Content-Type": "application/json"}).json():

            user_messages.append(Message(message))

        return user_messages


    def get_json_object(self) -> dict:

        return self.channel
    
    def send_message(self, content: str = "", embeds: list[Embed] = []) -> any:

        repsonse = HttpClient().request("POST", 'https://discordapp.com/api/v9//channels/{0}/messages'.format(self.id),
                            header={"Authorization": self.token, "Content-Type": "application/json"}, json={
                                'content': content,
                                'tts': False,
                                'embeds': embeds
                                })
        
        if repsonse.status_code != 200:

            return False
        
        return True



from .http_ import *
from .member import *


class Guild:

    """
    Класс представляющий информацию о гильдии.
    """

    def __init__(self, guild: dict, token: str):

        self.guild: dict = guild
        self.token: str = token

    @property
    def id(self) -> str:

        return self.guild['id']

    @property
    def name(self) -> str:

        return self.guild['name']

    @property
    def is_owner(self) -> bool:

        return self.guild['owner']

    def get_json_object(self) -> dict:

        return self.guild

    @property
    def members(self) -> list[Member]:

        guild_members: list[Member] = []

        for guild in HttpClient().request("GET", 'https://discordapp.com/api/v9/guilds/{0}/members'.format(self.id),
                                          {"Authorization": self.token, "Content-Type": "application/json"}).json():

            guild_members.append(Member(guild))

        return guild_members

    @property
    def members_count(self) -> int:

        return self.members.count



import requests


class HttpClient:

    """
    Класс для отправки запросов к API.
    """

    def __init__(self):
        ...

    def request(self, method: str, url: str, header: dict, json: dict = None) -> requests.Response:

        match method:

            case "POST":

                return self.post_request(url, header, json)

            case "GET":

                return self.get_request(url, header, json)

    def get_request(self, url: str, header: dict, json: dict = None) -> requests.Response:

        if json is not None:

            return requests.get(url=url, headers=header, json=json)

        return requests.get(url=url, headers=header)

    def post_request(self, url: str, header: dict, json: dict) -> requests.Response:

        return requests.post(url=url, headers=header, json=json)



from .channel import *
from .guild import *
from .http_ import *
from .message import *


class User:

    """
    Класс представляющий информацию о текущем подключенном объекте.
    """

    def __init__(self, user: dict, token: str):

        self.user: dict = user
        self.token: str = token

    @property
    def id(self) -> str:

        return self.user['id']

    @property
    def username(self) -> str:

        return self.user['username']

    @property
    def global_name(self) -> str:

        return self.user['global_name']

    @property
    def avatar(self) -> str:

        return self.user['avatar']

    @property
    def banner_color(self) -> str:

        return self.user['banner_color']


    def get_json_object(self) -> dict:

        return self.user

    @property
    def channels(self) -> list[Channel]:

        user_channels: list[Channel] = []

        for channel in HttpClient().request("GET", 'https://discordapp.com/api/v9/users/@me/channels',
                                            {"Authorization": self.token, "Content-Type": "application/json"}).json():

            user_channels.append(Channel(channel, self.token))

        return user_channels

    @property
    def guilds(self) -> list[Guild]:

        user_guilds: list[Guild] = []

        for guild in HttpClient().request("GET", 'https://discordapp.com/api/v9/users/@me/guilds',
                                          {"Authorization": self.token, "Content-Type": "application/json"}).json():

            user_guilds.append(Guild(guild, self.token))

        return user_guilds



from .eother import *


class Embed:

    def __init__(self, description: str = "", author: Author = None, fields: list[Field] = [], title: str = "", url: str = None, image: Image = None, thumbnail: Thumbnail = None, footer: Footer = None, timestamp: str = ""):

        self.description: str = description
        self.author: Author = author
        self.fields: list[Field] = fields
        self.title: str = title
        self.url: str = url
        self.image: Image = image
        self.footer: Footer = footer
        self.thumbnail: Thumbnail = thumbnail
        self.timestamp: str = timestamp

    def get_json(self) -> dict:

        return {
            "description": self.description,
            "fields": [field for field in self.fields],
            "author": self.get_author(),
            "title": self.title,
            "url": self.url,
            "image": self.get_image(),
            "thumbnail": self.get_thumbnail(),
            "timestamp": self.timestamp
        }

    def get_author(self) -> dict:

        if self.author is None:

            return {}

        return self.author.get_json()

    def get_image(self) -> dict:

        if self.image is None:

            return {}

        return self.image.get_json()

    def get_thumbnail(self) -> dict:

        if self.thumbnail is None:

            return {}

        return self.thumbnail.get_json()



import time
from threading import Thread

from .embed import *
from .http_ import *
from .message import *
from .user import *


class Client:

    """
    Класс выполняющий подключение с API.
    """

    def __init__(self):

        self.user: User = None

    def connect(self, token: str) -> any:

        response = HttpClient().request('GET', 'https://discordapp.com/api/v9/users/@me',
                                        {"Authorization": token, "Content-Type": "application/json"})

        if response.status_code != 200:

            raise ConnectionError

        self.user = User(response.json(), token)

    @property
    def is_ready(self) -> bool:

        return self.user is not None

    def on_ready(self, func):

        while self.is_ready is False:

            time.sleep(0.5)

        func()
        new_t = Thread(target=self.connect_stable)
        new_t.start()

    def get_messages(self) -> list[Message]:

        messages: list[Message] = []

        for channel in self.user.channels:

            messages += channel.messages

        return messages

    def message_processing(self, func):

        messages: list[Message] = self.get_messages()

        while True:

            for channel in self.user.channels:

                response = requests.get('https://discordapp.com/api/v9/channels/{0}/messages?after={1}&limit=5'.format(channel.id, messages[0].id),
                                        headers={"Authorization": self.user.token, "Content-Type": "application/json"}, )

                if response.json().count != 0:

                    for message in response.json():

                        func(Message(message))

                        messages.insert(0, Message(message))

            time.sleep(1)

    def send_message(self, content: str = "", embeds: list[Embed] = [], channel_id: str = "") -> any:

        repsonse = HttpClient().request("POST", 'https://discordapp.com/api/v9//channels/{0}/messages'.format(channel_id),
                                        header={"Authorization": self.user.token, "Content-Type": "application/json"}, json={
            'content': content,
            'tts': False,
            'embeds': embeds
        })

        if repsonse.status_code != 200:

            return False

        return True

    def connect_stable(self):

        while True:

            self.connect(self.user.token)
            time.sleep(5)

    def on_message(self, *func):

        while self.is_ready is False:

            time.sleep(0.5)

        new_t = Thread(target=self.message_processing, args=func)
        new_t.start()


class ConnectionError(Exception):

    ...


if __name__ == '__main__':

    ...



class Member:

    """
    Класс представляющий информацию о участнике.
    """

    def __init__(self, member: dict):

        self.member: dict = member

    @property
    def id(self) -> str:

        return self.member['id']

    @property
    def username(self) -> str:

        return self.member['username']

    @property
    def global_name(self) -> str:

        return self.member['global_name']

    @property
    def avatar(self) -> str:

        return self.member['avatar']

    def get_json_object(self) -> dict:

        return self.member



class Field:

    def __init__(self, name: str, value: str, inline: bool = False):

        self.name: str = name
        self.value: str = value
        self.inline: False = inline

    def get_json(self) -> dict:

        return {
            "name": self.name,
            "value": self.value,
            "inline": self.inline
        }


class Author:

    def __init__(self, name: str, url: str = None, icon_url: str = None):

        self.name: str = name
        self.url: str = url
        self.icon_url: str = icon_url

    def get_json(self) -> dict:

        return {
            "name": self.name,
            "url": self.url,
            "icon_url": self.icon_url
        }


class Image:

    def __init__(self, url: str):

        self.url: str = url

    def get_json(self) -> dict:

        return {
            "url": self.url
        }


class Thumbnail:

    def __init__(self, url: str):

        self.url: str = url

    def get_json(self) -> dict:

        return {
            "url": self.url
        }


class Footer:

    def __init__(self, text: str, icon_url: str):

        self.text: str = text
        self.icon_url: str = icon_url

    def get_json(self) -> dict:

        return {
            "text": self.text,
            "icon_url": self.icon_url
        }



from .member import *

class Message:

    def __init__(self, message: dict):

        self.message: dict = message
    
    def get_json_object(self) -> dict:

        return self.message
    
    @property
    def id(self) -> str:

        return str(self.message['id'])
    
    @property
    def channel_id(self) -> str:

        return str(self.message['channel_id'])
    
    @property
    def author(self) -> Member:

        return Member(self.message['author'])

    @property
    def content(self) -> str:

        return str(self.message['content'])


from setuptools import setup

setup(name='mentord',
      version='1.1.1',
      description='Library for discord self bots.',
      packages=['mentord'],
      long_description="""
### Example use:
```python
User_client: Client = Client() 
User_client.connect(token='YOUR TOKEN')
```

### Use decorators:
```python
@User_client.on_message
def client_wait_message(message: Message):
    print('new message!')
```
      """,
      long_description_content_type='text/markdown')